@@ -1,7 +1,7 @@
 /* pptp_ctrl.c ... handle PPTP control connection.
  *                 C. Scott Ananian <cananian@alumni.princeton.edu>
  *
- * $Id: pptp_ctrl.c,v 1.7 1997/12/16 15:20:06 cananian Exp $
+ * $Id: pptp_ctrl.c,v 1.10 2002/03/11 01:51:16 quozl Exp $
  */
 
 #include <errno.h>
@@ -21,6 +21,7 @@
 #include "pptp_options.h"
 #include "vector.h"
 #include "util.h"
+#include "pptp_quirks.h"
 
 /* BECAUSE OF SIGNAL LIMITATIONS, EACH PROCESS CAN ONLY MANAGE ONE
  * CONNECTION.  SO THIS 'PPTP_CONN' STRUCTURE IS A BIT MISLEADING.
@@ -118,6 +119,11 @@ int pptp_send_ctrl_packet(PPTP_CONN * conn, void * buffer, size_t size);
 void pptp_dispatch_packet(PPTP_CONN * conn, void * buffer, size_t size);
 /* Dispatch packets (control messages) */
 void pptp_dispatch_ctrl_packet(PPTP_CONN * conn, void * buffer, size_t size);
+/* Set link info, for pptp servers that need it.
+   this is a noop, unless the user specified a quirk and
+   there's a set_link hook defined in the quirks table
+   for that quirk */
+void pptp_set_link(PPTP_CONN * conn, int peer_call_id);
 /*----------------------------------------------------------------------*/
 /* Constructors and Destructors.                                        */
 
@@ -159,6 +165,14 @@ PPTP_CONN * pptp_conn_open(int inet_sock, int isclient, pptp_conn_cb callback)
       hton16(PPTP_MAX_CHANNELS), hton16(PPTP_FIRMWARE_VERSION), 
       PPTP_HOSTNAME, PPTP_VENDOR
     };
+
+    /* fix this packet, if necessary */
+    int idx, rc;
+    if ((idx = get_quirk_index()) != -1 && pptp_fixups[idx].start_ctrl_conn) {
+	if ((rc = pptp_fixups[idx].start_ctrl_conn(&packet)))
+	    warn("calling the start_ctrl_conn hook failed (%d)", rc);
+    }
+
     if (pptp_send_ctrl_packet(conn, &packet, sizeof(packet)))
       conn->conn_state = CONN_WAIT_CTL_REPLY;
     else return NULL; /* could not send initial start request. */
@@ -183,7 +197,8 @@ PPTP_CONN * pptp_conn_open(int inet_sock, int isclient, pptp_conn_cb callback)
  * We need to do something else to allocate calls for incoming requests.
  */
 PPTP_CALL * pptp_call_open(PPTP_CONN * conn,
-			   pptp_call_cb callback){
+			   pptp_call_cb callback, char *phonenr)
+{
   PPTP_CALL * call;
   int i;
   assert(conn && conn->call);
@@ -211,6 +226,23 @@ PPTP_CALL * pptp_call_open(PPTP_CONN * conn,
       hton32(PPTP_BEARER_CAP), hton32(PPTP_FRAME_CAP), 
       hton16(PPTP_WINDOW), 0, 0, 0, {0}, {0}
     };
+
+    int idx, rc;
+    /* if we have a quirk, build a new packet to fit it */
+    if ((idx = get_quirk_index()) != -1 && pptp_fixups[idx].out_call_rqst_hook) {
+	if ((rc = pptp_fixups[idx].out_call_rqst_hook(&packet)))
+	    warn("calling the out_call_rqst hook failed (%d)", rc);
+    }
+
+
+    /* fill in the phone number if it was specified */
+    if (phonenr) {
+        strncpy(packet.phone_num, phonenr, sizeof(packet.phone_num));
+        packet.phone_len = hton16 (strlen(phonenr));
+        if( packet.phone_len > sizeof(packet.phone_num))
+            packet.phone_len = sizeof(packet.phone_num);
+    }
+
     if (pptp_send_ctrl_packet(conn, &packet, sizeof(packet))) {
       pptp_reset_timer();
       call->state.pns = PNS_WAIT_REPLY;
@@ -297,13 +329,18 @@ void pptp_conn_destroy(PPTP_CONN * conn) {
 /************** Deal with messages, in a non-blocking manner *************/
 
 /* Add file descriptors used by pptp to fd_set. */
-void pptp_fd_set(PPTP_CONN * conn, fd_set * read_set, fd_set * write_set) {
+void pptp_fd_set(PPTP_CONN * conn, fd_set * read_set, fd_set * write_set,
+                 int *max_fd) {
   assert(conn && conn->call);
+
   /* Add fd to write_set if there are outstanding writes. */
   if (conn->write_size > 0)
     FD_SET(conn->inet_sock, write_set);
+
   /* Always add fd to read_set. (always want something to read) */
   FD_SET(conn->inet_sock, read_set);
+  if (*max_fd < conn->inet_sock)
+    *max_fd = conn->inet_sock;
 }
 /* handle any pptp file descriptors set in fd_set, and clear them */
 void pptp_dispatch(PPTP_CONN * conn, fd_set * read_set, fd_set * write_set) {
@@ -384,7 +421,9 @@ int pptp_make_packet(PPTP_CONN * conn, void **buf, size_t *size) {
     /* Throw out bytes until we have a valid header. */
     header = (struct pptp_header *) (conn->read_buffer+bad_bytes);
     if (ntoh32(header->magic) != PPTP_MAGIC) goto throwitout;
-    if (ntoh16(header->reserved0) != 0) goto throwitout;
+    if (ntoh16(header->reserved0) != 0)
+	log("reserved0 field is not zero! (0x%x) Cisco feature? \n",
+		ntoh16(header->reserved0));
     if (ntoh16(header->length) < sizeof(struct pptp_header)) goto throwitout;
     if (ntoh16(header->length) > PPTP_CTRL_SIZE_MAX) goto throwitout;
     /* well.  I guess it's good. Let's see if we've got it all. */
@@ -400,7 +439,7 @@ int pptp_make_packet(PPTP_CONN * conn, void **buf, size_t *size) {
     *size= ntoh16(header->length);
     *buf = malloc(*size);
     if (*buf == NULL) { log("Out of memory."); return 0; /* ack! */ }
-    memcpy(*buf, conn->read_buffer, *size);
+    memcpy(*buf, conn->read_buffer+bad_bytes, *size);
     /* Delete this packet from the read_buffer. */
     conn->read_size -= (bad_bytes + *size);
     memmove(conn->read_buffer, conn->read_buffer+bad_bytes+*size, 
@@ -453,7 +492,7 @@ void pptp_dispatch_packet(PPTP_CONN * conn, void * buffer, size_t size) {
     break;
   default:
     log("Unknown PPTP control message type received: %u", 
-	(unsigned) ntoh16(header->pptp_type));
+	(unsigned int) ntoh16(header->pptp_type));
     break;
   }
 }
@@ -485,6 +524,14 @@ void pptp_dispatch_ctrl_packet(PPTP_CONN * conn, void * buffer, size_t size) {
 	hton32(PPTP_FRAME_CAP), hton32(PPTP_BEARER_CAP),
 	hton16(PPTP_MAX_CHANNELS), hton16(PPTP_FIRMWARE_VERSION),
 	PPTP_HOSTNAME, PPTP_VENDOR };
+
+      /* fix this packet, if necessary */
+      int idx, rc;
+      if ((idx = get_quirk_index()) != -1 && pptp_fixups[idx].start_ctrl_conn) {
+	  if ((rc = pptp_fixups[idx].start_ctrl_conn(&reply)))
+	      warn("calling the start_ctrl_conn hook failed (%d)", rc);
+      }
+
       if (conn->conn_state == CONN_IDLE) {
 	if (ntoh16(packet->version) < PPTP_VERSION) {
 	  /* Can't support this (earlier) PPTP_VERSION */
@@ -513,8 +560,10 @@ void pptp_dispatch_ctrl_packet(PPTP_CONN * conn, void * buffer, size_t size) {
 	  close_reason = PPTP_STOP_PROTOCOL;
 	  goto pptp_conn_close;
 	}
-	if (ntoh8(packet->result_code)!=1) { /* some problem with start */
-	  /* if result_code == 5, we might fall back to different version */
+        /* J'ai change le if () afin que la connection ne se ferme pas pour un "rien" :p
+         * adel@cybercable.fr -
+         */                                            
+	if (ntoh8(packet->result_code) !=1 && ntoh8(packet->result_code) !=0) { 
 	  if (conn->callback!=NULL) conn->callback(conn, CONN_OPEN_FAIL);
 	  close_reason = PPTP_STOP_PROTOCOL;
 	  goto pptp_conn_close;
@@ -537,8 +586,10 @@ void pptp_dispatch_ctrl_packet(PPTP_CONN * conn, void * buffer, size_t size) {
   /* ----------- STANDARD Stop-Session MESSAGES ------------ */
   case PPTP_STOP_CTRL_CONN_RQST:
     {
+#if 0 /* make gcc happy about "unused variable 'packet'" here */
       struct pptp_stop_ctrl_conn *packet = /* XXX do something with this XXX */
 	(struct pptp_stop_ctrl_conn *) buffer; 
+#endif
       /* conn_state should be CONN_ESTABLISHED, but it could be 
        * something else */
       struct pptp_stop_ctrl_conn reply = {
@@ -555,8 +606,10 @@ void pptp_dispatch_ctrl_packet(PPTP_CONN * conn, void * buffer, size_t size) {
     }
   case PPTP_STOP_CTRL_CONN_RPLY:
     {
+#if 0 /* make gcc happy about "unused variable 'packet'" here */
       struct pptp_stop_ctrl_conn *packet = /* XXX do something with this XXX */
 	(struct pptp_stop_ctrl_conn *) buffer;
+#endif
       /* conn_state should be CONN_WAIT_STOP_REPLY, but it 
        * could be something else */
 
@@ -622,8 +675,21 @@ void pptp_dispatch_ctrl_packet(PPTP_CONN * conn, void * buffer, size_t size) {
       if (call->state.pns == PNS_WAIT_REPLY) {
 	/* check for errors */
 	if (packet->result_code!=1) {
-	  /* An error.  Log it. */
-	  log("Error opening call. [callid %d]", (int) callid);
+	  /* An error.  Log it verbosely. */
+	  unsigned int legal_error_value =
+	       sizeof(pptp_general_errors)/sizeof(pptp_general_errors[0]);
+	  int err = packet->error_code;
+          log("Error '%d' opening call. [callid %d]",
+	      packet->result_code, (int) callid);
+          log("Error code is '%d', Cause code is '%d'", err,
+	      packet->cause_code);
+          if ((err > 0) && (err < legal_error_value)){
+            log("Error is '%s', Error message: '%s'",
+            pptp_general_errors[err].name,
+            pptp_general_errors[err].desc);
+	  }
+
+
 	  call->state.pns = PNS_IDLE;
 	  if (call->callback!=NULL) call->callback(conn, call, CALL_OPEN_FAIL);
 	  pptp_call_destroy(conn, call);
@@ -633,8 +699,13 @@ void pptp_dispatch_ctrl_packet(PPTP_CONN * conn, void * buffer, size_t size) {
 	  call->peer_call_id = ntoh16(packet->call_id);
 	  call->speed        = ntoh32(packet->speed);
 	  pptp_reset_timer();
+	  /* call pptp_set_link. unless the user specified a quirk
+	     and this quirk has a set_link hook, this is a noop */
+ 	  pptp_set_link(conn, call->peer_call_id);
+
 	  if (call->callback!=NULL) call->callback(conn, call, CALL_OPEN_DONE);
-	  log("Outgoing call established.\n");
+	  log("Outgoing call established (call ID %u, peer's call ID %u).\n",
+		call->call_id, call->peer_call_id);
 	}
       }
       break;
@@ -683,7 +754,7 @@ void pptp_dispatch_ctrl_packet(PPTP_CONN * conn, void * buffer, size_t size) {
 	break; /* this is what we expect. */
       /* log it, otherwise. */
       log("PPTP_SET_LINK_INFO recieved from peer_callid %u",
-	  (unsigned) ntoh16(packet->call_id_peer));
+	  (unsigned int) ntoh16(packet->call_id_peer));
       log("  send_accm is %08lX, recv_accm is %08lX",
 	  (unsigned long) ntoh32(packet->send_accm),
 	  (unsigned long) ntoh32(packet->recv_accm));
@@ -702,6 +773,23 @@ pptp_conn_close:
   pptp_conn_close(conn, close_reason);
 }
 
+void
+pptp_set_link(PPTP_CONN* conn, int peer_call_id)
+{
+    int idx, rc;
+
+    /* if we need to send a set_link packet because of buggy
+       hardware or pptp server, do it now */
+    if ((idx = get_quirk_index()) != -1 && pptp_fixups[idx].set_link_hook) {
+	struct pptp_set_link_info packet;
+	if ((rc = pptp_fixups[idx].set_link_hook(&packet, peer_call_id)))
+	    warn("calling the set_link hook failed (%d)", rc);
+
+	if (pptp_send_ctrl_packet(conn, &packet, sizeof(packet))) {
+	    pptp_reset_timer();
+	}
+    }
+}
 
 /********************* Get info from call structure *******************/
 
