@@ -1,24 +1,25 @@
 /* util.c ....... error message utilities.
  *                C. Scott Ananian <cananian@alumni.princeton.edu>
  *
- * $Id: util.c,v 1.1 1997/12/13 08:39:02 cananian Exp $
+ * $Id: util.c,v 1.4 2002/03/11 01:50:07 quozl Exp $
  */
 
 #include <stdio.h>
 #include <stdarg.h>
 #include <syslog.h>
 #include <unistd.h>
+#include <stdlib.h>
 #include "util.h"
 
 #ifndef PROGRAM_NAME
-#define PROGRAM_NAME "(unknown)"
+#define PROGRAM_NAME "pptp"
 #endif
 
 static void open_log(void) __attribute__ ((constructor));
 static void close_log(void) __attribute__ ((destructor));
 
 static void open_log(void) {
-  openlog(PROGRAM_NAME, LOG_PERROR | LOG_PID, LOG_DAEMON);
+  openlog(PROGRAM_NAME, LOG_PID, LOG_DAEMON);
 }
 static void close_log(void) {
   closelog();
@@ -41,10 +42,12 @@ void _log(char *func, char *file, int line, char *format, ...) {
 void _warn(char *func, char *file, int line, char *format, ...) {
   MAKE_STRING("warn");
   fprintf(stderr, "%s\n", string);
+  syslog(LOG_NOTICE, "%s", string);
 }
 
 void _fatal(char *func, char *file, int line, char *format, ...) {
   MAKE_STRING("fatal");
   fprintf(stderr, "%s\n", string);
+  syslog(LOG_NOTICE, "%s", string);
   exit(1);
 }
